"""
Does something.
"""
import argparse
from datetime import datetime

import numpy as np
from bioplottemplates.plots import param

from taurenmd import Path, log  # noqa: F401
from taurenmd.libs import libcalc, libcli, libmda, libutil
from taurenmd.logger import S, T


_REF_FRAME = 0


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-x',
    '--export',
    help=(
        'Export calculated RMSDs to CSV file. '
        'If given defaults to rmsd.csv, alternatively, '
        'you can give a specific file name.'
        ),
    default=False,
    const='rmsd.csv',
    nargs='?',
    )

ap.add_argument(
    '-l',
    '--selections',
    help=(
        'The atom selection upon which calculate the RMSDs. '
        'You can give multiple selections to calculate multiple RMSDs sets. '
        'Defauts to \'all\'.'
        ),
    type=str,
    default=None,
    nargs='+',
    )

ap.add_argument(
    '-r',
    '--ref-frame',
    help='The reference frame.',
    type=int,
    default=_REF_FRAME,
    )

ap.add_argument(
    '-v',
    '--plotvars',
    help=(
        'Plot variables. '
        'Example: -v xlabel=frames ylabel=RMSD color=red.'
        ),
    nargs='*',
    action=libcli.ParamsToDict,
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    input(cmd)
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        start=None,
        stop=None,
        step=None,
        ref_frame=_REF_FRAME,
        selections=None,
        plotvars=None,
        export=False,
        **kwargs
        ):
    
    log.info(T('starting'))
    
    u = libmda.mda_load_universe(topology, *list(trajectory))
    
    frame_slice = libutil.frame_slice(
        start=start,
        stop=stop,
        step=step,
        )
    
    if selections is None:
        selections = ['all']
    rmsds = []
    for selection in selections:
        rmsds.append(
            libcalc.mda_rmsd_combined_chains(
                u,
                frame_slice=frame_slice,
                selection=selection,
                ref_frame=ref_frame,
                )
            )
    if export:
        np.savetxt(
            export,
            np.array([range(len(u.trajectory))[frame_slice]] + rmsds).T,
            fmt=['%d'] + ['%.5e'] * len(rmsds),
            delimiter=',',
            newline='\n',
            header=(
                "Date: {}\n'"
                "Topology: {}\n"
                "Trajectory : {}\n"
                "ref frame: {}\n"
                "frame number, {}\n"
                ).format(
                    datetime.now(),
                    Path(topology).resolve(),
                    [Path(f).resolve().str() for f in trajectory],
                    ','.join(selections),
                    ref_frame,
                    ),
            footer='',
            comments='#',
            encoding=None,
            )

    if plotvars is None:
        plotvars = dict()
    
    if 'labels' not in plotvars:
        plotvars['labels'] = selections

    log.info(T('plot params:'))
    for k, v in plotvars.items():
        log.info(S('{} = {!r}', k, v))
    
    param.plot(
        list(range(len(u.trajectory))[frame_slice]),
        rmsds,
        **plotvars,
        )

    return


if __name__ == '__main__':
    maincli()



"""Common operations for client interfaces."""
import argparse
import sys
from datetime import datetime


# https://stackoverflow.com/questions/4042452
class CustomParser(argparse.ArgumentParser):
    """Custom Parser class."""
    
    def error(self, message):
        """Present error message."""
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


class ParamsToDict(argparse.Action):
    """Convert command-line parameters to dict."""
    
    def __call__(self, parser, namespace, values, option_string=None):
        """Executes."""

        bool_value = {
            'true': True,
            'false': False,
            }

        param_dict = {}
        for kv in values:
            # print(param_dict, kv)
            try:
                k, v = kv.split('=')
            except ValueError:
                param_dict[kv] = True
            else:
                if ',' in v:
                    vs = v.split(',')
                    try:
                        param_dict[k] = tuple(float(i) for i in vs)
                    except (ValueError, TypeError):
                        param_dict[k] = tuple(i for i in vs)

                else:
                    try:
                        param_dict[k] = float(v)
                    except (ValueError, TypeError):  # is string or list
                        param_dict[k] = bool_value.get(v.lower(), v)
            
        setattr(namespace, self.dest, param_dict)


def save_command(fname, *args):
    """Append the input command to a file."""
    with open(fname, 'a') as fh:
        fh.write(
            '[{}] {}\n'.format(
                datetime.now().strftime("%d/%m/%Y, %H:%M:%S"),
                ' '.join(args),
                )
            )



"""
Handles input and output.
"""
from taurenmd import Path, log
from taurenmd.logger import S, T  # noqa: F401


def _get_number(path):
    return int(Path(path).stem.split('_')[-1])


def sort_numbered_input(*inputs):
    """
    Sort input paths to tail number.

    Input paths or strings should be formatted such:
        
        my_trajectory_#.dcd

    where # is the tag evaluated to int and input sorted accordingly.

    Returns
    -------
    list
        The sorted pathlist
    """
    return sorted(inputs, key=_get_number)
   

def report_input(topology, trajectory):
    log.info(S('loading trajectory: {}', trajectory))
    log.info(S('with topology: {}', topology))
    return


def get_stem(filename, ext=None):
    """
    Returns the stem name of a file name.
    
    Example:
        >>> get_stem('trajectory.dcd', ext='pdb')
        >>> trajectory.pdb
    
    Parameters
    ----------
    filename
        The file name.
    """
    
    output = Path(filename).stem

    if ext:
        output = Path(output).with_suffix('.' + ext.lstrip("."))

    return output


def mk_frame_path(input_path, frame=0, ext='.pdb'):
    """
    Create the path name for a frame.

    Given an input_path (usually the name of the trajectory), create
    the corresponding frame filename.

    Example:
        >>> mk_frame_path('traj_output.xtc')
        >>> traj_output_frame0.pdb
    """
    input_path = Path(input_path)
    top_output = Path(
        input_path.myparents(),
        '{}_frame{}'.format(input_path.stem, frame),
        ).with_suffix('.{}'.format(ext.lstrip('.')))

    return top_output



"""
Calculates RMSFS of a selection along the trajectory.

Uses `MDAnalysis.analysis.rms.RMSF`_, read their documentation for details.

.. _MDAnalysis.analysis.rms.RMSF: https://www.mdanalysis.org/docs/documentation_pages/analysis/rms.html?highlight=rmsf#MDAnalysis.analysis.rms.RMSF
"""  # noqa: E501
import argparse
from datetime import datetime

from bioplottemplates.plots import label_dots

from taurenmd import Path, log  # noqa: F401
from taurenmd.libs import libcalc, libcli, libmda, libutil
from taurenmd.logger import S, T


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-x',
    '--export',
    help=(
        'Export calculated RMSFs to CSV file. '
        'If given defaults to rmsf.csv, alternatively, '
        'you can give a specific file name.'
        ),
    default=False,
    const='rmsf.csv',
    nargs='?',
    )

ap.add_argument(
    '-l',
    '--selection',
    help='The atom selection upon which calculate the RMSFs.',
    type=str,
    default=None,
    )

ap.add_argument(
    '-v',
    '--plotvars',
    help=(
        'Plot variables. '
        'Example: -v xlabel=frames ylabel=RMSD color=red.'
        ),
    nargs='*',
    action=libcli.ParamsToDict,
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    input(cmd)
    return cmd


def maincli():
    """Main client call."""
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        start=None,
        stop=None,
        step=None,
        selection=None,
        plotvars=None,
        export=False,
        **kwargs
        ):
    """Execute client main logic."""
    log.info(T('starting'))
   
    u = libmda.mda_load_universe(topology, *list(trajectory))
   
    frame_slice = libutil.frame_slice(
        start=start,
        stop=stop,
        step=step,
        )
   
    selection = selection or 'all'
  
    labels = []
    rmsfs = []

    atom_group = u.select_atoms(selection)
    labels = libmda.draw_atom_label_from_atom_group(atom_group)

    log.info(S('for selection: {}', selection))
    rmsfs = libcalc.mda_rmsf(
        atom_group,
        frame_slice=frame_slice,
        )

    if export:
        header = (
            "# Date: {}\n"
            "# Topology: {}\n"
            "# Trajectory : {}\n"
            "# Atom,RMSF\n"
            ).format(
                datetime.now().strftime("%d/%m/%Y, %H:%M:%S"),
                Path(topology).resolve(),
                [Path(f).resolve().str() for f in trajectory],
                )
        with open(export, 'w') as fh:
            fh.write(header)
            for label, value in zip(labels, rmsfs):
                fh.write('{},{:.5}\n'.format(label, value))

    plotvars = plotvars or dict()
    plotvars.setdefault('series_labels', selection)

    log.info(T('plot params:'))
    for k, v in plotvars.items():
        log.info(S('{} = {!r}', k, v))
    
    label_dots.plot(
        labels,
        rmsfs,
        **plotvars,
        )

    return


if __name__ == '__main__':
    maincli()




import taurenmd.cli_imagemol as cli_imagemol  # noqa: F401
import taurenmd.cli_noSol as cli_noSol  # noqa: F401
import taurenmd.cli_report as cli_report  # noqa: F401
import taurenmd.cli_rmsd as cli_rmsd  # noqa: F401
import taurenmd.cli_template as cli_template  # noqa: F401
import taurenmd.cli_trajedit as cli_trajedit  # noqa: F401


def test_main():
    cli_template.main('dummy_topo', 'dummy_traj')



"""
Does something.
"""
import argparse
import copy

import numpy as np
from bioplottemplates.plots import param

from taurenmd import log
from taurenmd.libs import libcli, libio, libmda, libutil  # noqa: F401
from taurenmd.logger import S, T


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    # formatter_class=argparse.RawDescriptionHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-l1',
    '--sel1',
    help='First selection.',
    default='all',
    type=str,
    )

ap.add_argument(
    '-l2',
    '--sel2',
    help='Second selection.',
    default='all',
    type=str,
    )

ap.add_argument(
    '-f',
    '--frame',
    help='Calc distances to a frame instead of relative along traj.',
    default=None,
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-v',
    '--plotvars',
    help=(
        'Plot variables. '
        'Example: -v xlabel=frames ylabel=RMSD color=red.'
        ),
    nargs='*',
    action=libcli.ParamsToDict,
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        sel1='all',
        sel2='all',
        start=None,
        stop=None,
        step=None,
        frame=None,
        plotvars=None,
        **kwargs
        ):
    log.info(T('measuring distances'))

    u = libmda.mda_load_universe(topology, *list(trajectory))

    frame_slice = libutil.frame_slice(
        start=start,
        stop=stop,
        step=step,
        )
    log.info(S('for slice {}', frame_slice))
   
    log.info(T('defining atom seletions'))
    log.info(S('atom selection #1: {}', sel1))
    log.info(S('atom selection #2: {}', sel2))
    atom_sel1 = u.select_atoms(sel1)
    atom_sel2 = u.select_atoms(sel2)

    distances = np.ones(len(u.trajectory[frame_slice]), dtype=np.float32)
    
    if frame is not None:
        u.trajectory[int(frame)]
        reference_cog = copy.deepcopy(atom_sel1.center_of_geometry())

    log.info(T('Calculating distances'))
    # https://www.mdanalysis.org/MDAnalysisTutorial/atomgroups.html
    # https://www.mdanalysis.org/docs/documentation_pages/core/groups.html#MDAnalysis.core.groups.AtomGroup.center_of_geometry
    for i, ts in enumerate(u.trajectory[frame_slice]):

        if frame is None:
            reference_cog = atom_sel1.center_of_geometry()

        distances[i] = np.linalg.norm(
            np.subtract(
                reference_cog,
                atom_sel2.center_of_geometry(),
                )
            )
    
    log.info(S('calculated a total of {} distances.', len(distances)))
    
    if plotvars is None:
        plotvars = dict()
    
    if 'labels' not in plotvars:
        plotvars['labels'] = '{} dist {}'.format(sel1, sel2)

    log.info(T('plot params:'))
    for k, v in plotvars.items():
        log.info(S('{} = {!r}', k, v))

    param.plot(
        list(range(len(u.trajectory))[frame_slice]),
        distances,
        **plotvars,
        )

    log.info(S('done'))
    return


if __name__ == '__main__':
    maincli()



"""
Input/Output functions using MDAnalysis.
"""
import MDAnalysis as mda

from taurenmd import log
from taurenmd.libs import libio
from taurenmd.logger import S, T


def mda_load_universe(topology, *trajectories):
    """
    Load MDAnalysis universe.

    Parameters
    ----------
    top
        Topology file.

    traj
        Trajectory file.
    
    Return
    ------
    MDAnalysis Universe
    """
    libio.report_input(topology, trajectories)

    universe = mda.Universe(topology, trajectories)

    mda_report(universe)
    return universe


def mda_report(universe):
    """Report information about the Universe."""
    log.info(T('Reporting'))
    log.info(S('number of frames: {}', len(universe.trajectory)))
    log.info(S('number of atoms: {}', len(universe.atoms)))


def draw_atom_label_from_atom_group(atom_group):
    """
    Translate `atom_group` to list of representing strings
    for each atom.
    """

    labels = []
    for atom in atom_group:
        s = '{}.{}{}.{}'.format(
            atom.segment.segid,
            atom.residue.resnum,
            atom.resname,
            atom.name,
            )
        labels.append(s)

    return labels



"""
Edit a trajectory.

File format, length (that is, frames) and selection can be edited.

Uses MDAnalsysis.
"""
import argparse

import MDAnalysis as mda
from MDAnalysis.analysis import align as mdaalign

from taurenmd import Path, log
from taurenmd.libs import libio, libmda
from taurenmd.logger import S, T


ap = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-i',
    '--insort',
    help=(
        'Sorts input trajectory paths according to their tail numbers, '
        'if paths are formatted: my_trajectory_#.dcd, '
        'where # is a number.'
        ),
    action='store_true',
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-l',
    '--selection',
    help=(
        'Atom selection for the output trajectory. '
        'Read: https://www.mdanalysis.org/docs/documentation_pages/selections.html'  # noqa: E501
        ),
    default='all',
    type=str,
    )

ap.add_argument(
    '-d',
    '--traj-output',
    help='Edited trajectory. Defaults to traj_output.xtc.',
    default='traj_output.xtc',
    type=Path,
    )

ap.add_argument(
    '-o',
    '--top-output',
    help=(
        'Topology output first frame.'
        'Defaults to --traj-output file name + _frame0.'
        ),
    default=None,
    )

ap.add_argument(
    '-O',
    '--save-frame0-topology',
    help=(
        'Oposite of -o. '
        'Do NOT save frame0 as topology. '
        'Defaults to False, that is, saves topology. '
        ),
    action='store_false',
    )

ap.add_argument(
    '-a',
    '--align',
    help='Align selection (l) to a atom subselection',
    action='store_true',
    )

ap.add_argument(
    '--align-selection',
    help=(
        'The reference atom group to which align the trajectory to. '
        'Must be subselection of --selection.'
        ),
    type=str,
    default='all',
    )

ap.add_argument(
    '-w',
    '--unwrap',
    help=(
        'Unwraps selection according to: '
        'https://www.mdanalysis.org/docs/documentation_pages/core/groups.html#MDAnalysis.core.groups.AtomGroup.unwrap'  # noqa: E501
        ),
    action='store_true',
    )

ap.add_argument(
    '--unwrap-reference',
    help=(
        'The unwrap method reference parameter. '
        'Has effect only if \'-w\' is given. '
        'Defaults to `None`.'
        ),
    default=None,
    )

ap.add_argument(
    '--unwrap-compound',
    help=(
        'The unwrap method compound parameter. '
        'Has effect only if \'-w\' is given. '
        'Defaults to `fragments`.'
        ),
    default='fragments',
    type=str,
    )


def load_args():
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        insort=None,
        start=None,
        stop=None,
        step=None,
        selection='all',
        traj_output='traj_output.xtc',
        top_output=None,
        save_frame0_topology=True,
        unwrap=False,
        unwrap_reference=None,
        unwrap_compound='fragments',
        align=False,
        align_selection='all',
        **kwargs,
        ):
   
    log.info(T('editing trajectory'))
    
    if insort:
        trajectories = libio.sort_numbered_input(*list(trajectory))
    else:
        trajectories = list(trajectory)

    u = libmda.mda_load_universe(topology, trajectories)
    
    if unwrap:
        log.info(T('unwrapping'))
        log.info(S('set to: {}', unwrap))
        log.info(S('reference: {}', unwrap_reference))
        log.info(S('compound: {}', unwrap_compound))

    if align:
        u_top = mda.Universe(topology).select_atoms(selection)
        log.info(T('Alignment'))
        log.info(S('trajectory selection will be aligned to subselection:'))
        log.info(S('- {}', align_selection, indent=2))
    
    log.info(T('transformation'))
    log.info(S('slicing: {}::{}::{}', start, stop, step))
    sliceObj = slice(start, stop, step)

    log.info(S('selecting: {}', selection))
    selection = u.select_atoms(selection)
    log.info(S('with {} atoms', selection.n_atoms, indent=2))

    log.info(T('saving trajectory'))
    traj_output = Path(traj_output)
    log.info(S('destination: {}', traj_output.resolve().str()))

    with mda.Writer(traj_output.str(), selection.n_atoms) as W:
        for i, ts in zip(
                range(len(u.trajectory))[sliceObj],
                u.trajectory[sliceObj],
                ):
            
            log.info(S('working on frame: {}', i))
            
            if unwrap:
                log.debug(S('unwrapping', indent=2))
                selection.unwrap(
                    reference=unwrap_reference,
                    compound=unwrap_compound,
                    )

            if align:
                mdaalign.alignto(
                    selection,
                    u_top,
                    select=align_selection,
                    )

            W.write(selection)
    
    log.info(S('trajectory saved'))

    if save_frame0_topology:
        
        log.info(T('saving topology'))

        if top_output is None:
            top_output = libio.mk_frame_path(traj_output)
        else:
            top_output = Path(top_output)
        
        log.info(S('saving frame 0 to: {}', top_output.resolve()))
        with mda.Writer(Path(top_output).str(), selection.n_atoms) as W:
            for ts in u.trajectory[sliceObj][0:1]:
                if unwrap:
                    log.debug(S('unwrapping for topology', indent=2))
                    selection.unwrap(
                        reference=unwrap_reference,
                        compound=unwrap_compound,
                        )
                W.write(selection)
    
    log.info(S('Done'))
    return


if __name__ == '__main__':
    maincli()



"""
Report on trajectory characteristics.
"""
import argparse

from taurenmd import log
from taurenmd.libs import libcli, libmda


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.RawDescriptionHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help='The trajectory file',
    nargs='+',
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(topology, trajectory, **kwargs):
    log.info('Starting...')
    
    libmda.mda_load_universe(topology, *list(trajectory))
    return


if __name__ == '__main__':
    maincli()



"""
Extracts frames to PDB files.
"""
import argparse

from taurenmd import Path, log
from taurenmd.libs import libcli, libmda, libutil
from taurenmd.logger import S


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    # formatter_class=argparse.RawDescriptionHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-t',
    '--flist',
    help='List of frames (time steps) to extract.',
    default=False,
    nargs='+',
    )

ap.add_argument(
    '-f',
    '--prefix',
    help='String prefix for each file. Defaults to `frame_`.',
    default='frame_',
    )

ap.add_argument(
    '-x',
    '--ext',
    help='Extension of frame files. Defaulst to .pdb',
    default='.pdb',
    )

ap.add_argument(
    '-l',
    '--selection',
    help=(
        'Atom selection for the output trajectory. '
        'Read: https://www.mdanalysis.org/docs/documentation_pages/selections.html'  # noqa: E501
        ),
    default='all',
    type=str,
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        start=None,
        stop=None,
        step=None,
        flist=None,
        prefix='frame_',
        ext='pdb',
        selection='all',
        **kwargs):
    log.info('Starting...')
    
    u = libmda.mda_load_universe(topology, *list(trajectory))
    
    frames_to_extract = libutil.frame_list(
        len(u.trajectory),
        start=start,
        stop=stop,
        step=step,
        flist=flist,
        )

    log.info(S('extracting {} frames', len(frames_to_extract)))
    
    zeros = len(str(len(u.trajectory)))
    ext = ext.lstrip('.').strip()
    
    atom_group = u.select_atoms(selection)

    for frame in frames_to_extract:
        file_name = '{}{}.{}'.format(
            prefix,
            str(frame).zfill(zeros),
            ext,
            )

        atom_group.write(
            filename=Path(file_name),
            frames=[frame],
            )

        log.info(S('writen frame {}, to {}', frame, file_name))

    return


if __name__ == '__main__':
    maincli()



"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -mtaurenmd` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``taurenmd.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``taurenmd.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""
import sys

import taurenmd.cli_angle as cli_angle
import taurenmd.cli_distances as cli_dist
import taurenmd.cli_fext as cli_fext
import taurenmd.cli_imagemol as cli_imagemol
import taurenmd.cli_noSol as cli_noSol
import taurenmd.cli_report as cli_report
import taurenmd.cli_rmsd as cli_rmsd
import taurenmd.cli_rmsf as cli_rmsf
import taurenmd.cli_trajedit as cli_trajedit
from taurenmd import CMDFILE, log
from taurenmd.libs import libcli


def load_args():
    
    ap = libcli.CustomParser(description=__doc__)

    subparsers = ap.add_subparsers(title='TAURENMD SUBROUTINES')

    ap_angle = subparsers.add_parser(
        'angle',
        help='Calculates the angle between a plane along the trajectory.',
        parents=[cli_angle.ap],
        add_help=False,
        )
    ap_angle.set_defaults(func=cli_angle.main)

    ap_dist = subparsers.add_parser(
        'dist',
        help='Calculates distances between geometric centres of selections',
        parents=[cli_dist.ap],
        add_help=False,
        )
    ap_dist.set_defaults(func=cli_dist.main)
    
    ap_imagemol = subparsers.add_parser(
        'imagemol',
        help='Attempts to image molecules.',
        parents=[cli_imagemol.ap],
        add_help=False,
        )
    ap_imagemol.set_defaults(func=cli_imagemol.main)
    
    ap_fext = subparsers.add_parser(
        'fext',
        help='Extracts single frames from trajectory.',
        parents=[cli_fext.ap],
        add_help=False,
        )
    ap_fext.set_defaults(func=cli_fext.main)
    
    ap_noSol = subparsers.add_parser(
        'noSol',
        help='Removes solvent and extracts fist frame',
        parents=[cli_noSol.ap],
        add_help=False,
        )
    ap_noSol.set_defaults(func=cli_noSol.main)
    
    ap_tedit = subparsers.add_parser(
        'trajedit',
        help='Converts traj to PDB.',
        parents=[cli_trajedit.ap],
        add_help=False,
        )
    ap_tedit.set_defaults(func=cli_trajedit.main)

    ap_report = subparsers.add_parser(
        'report',
        help='Reports on traj details.',
        parents=[cli_report.ap],
        add_help=False,
        )
    ap_report.set_defaults(func=cli_report.main)

    ap_rmsd = subparsers.add_parser(
        'rmsd',
        help='Calculates and plot RMSDs',
        parents=[cli_rmsd.ap],
        add_help=False,
        )
    ap_rmsd.set_defaults(func=cli_rmsd.main)

    ap_rmsf = subparsers.add_parser(
        'rmsf',
        help='Calculates and plot RMSFs',
        parents=[cli_rmsf.ap],
        add_help=False,
        )
    ap_rmsf.set_defaults(func=cli_rmsf.main)

    if len(sys.argv) < 2:
        ap.print_help()
        ap.exit()

    cmd = ap.parse_args()

    return cmd


def maincli():
    libcli.save_command(CMDFILE, *sys.argv)
    args = load_args()
    log.debug(args)
    args.func(**vars(args))


if __name__ == '__main__':
    maincli()



"""
Attempt image molecule with mdtraj.
"""

from taurenmd import Path, log
from taurenmd.libs import libcli, libio, libmdt
from taurenmd.logger import S, T


_TRAJOUTPUT = 'production_imaged.dcd'


ap = libcli.CustomParser()

ap.add_argument(
    'topology',
    help='The topology file',
    )

ap.add_argument(
    'trajectory',
    help='The trajectory file.',
    )

ap.add_argument(
    '-d',
    '--traj-output',
    help='Output trajectory. Defaults to production_imaged.xtc.',
    default=_TRAJOUTPUT,
    )

ap.add_argument(
    '-o',
    '--top-output',
    help=(
        'File name to save the first frame of the imaged trajectory.'
        ' Defaults to the --traj-output path + \'_frame0.pdb.'
        ),
    default=None,
    )

ap.add_argument(
    '-p',
    '--protocol',
    help='The protocol with which reimage.',
    default=1,
    type=int,
    )


def load_args():
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))


def protocol1(traj):
    """
    Attempts to image molecules acting on the whole traj.
    """
    log.info(T('running reimage protocol #1'))
    log.info(S('finding molecules'))

    mols = traj.top.find_molecules()
    log.info(S('done'))
    
    log.info(T('reimaging'))
    reimaged = traj.image_molecules(
        inplace=False,
        anchor_molecules=mols[:1],
        other_molecules=mols[1:],
        )
    log.info(S('done'))
    return reimaged


def protocol2(traj):
    """
    Attempts to image molecules frame by frame.

    .. note::

        Have not found a use for this protocol yet.

    """
    reimaged = []
    for frame in range(len(traj)):
        log.info(S('reimaging frame: {}', frame))
        
        mols = traj[frame].top.find_molecules()
    
        reimaged.append(
            traj[frame].image_molecules(
                inplace=False,
                anchor_molecules=mols[:1],
                other_molecules=mols[1:],
                )
            )

    log.info(S('concatenating traj frames'))
    # http://mdtraj.org/1.9.3/api/generated/mdtraj.join.html#mdtraj.join
    reimaged_traj = reimaged[0].join(reimaged[1:])

    return reimaged_traj


def main(
        topology,
        trajectory,
        traj_output=_TRAJOUTPUT,
        top_output=None,
        protocol=1,
        **kwargs
        ):

    log.info('Attempting image molecules')
    
    traj = libmdt.mdtraj_load_traj(topology, trajectory)
    
    protocols = {
        1: protocol1,
        2: protocol2,
        }

    reimaged = protocols[protocol](traj)

    log.info(T('saving the output'))
    reimaged.save_dcd(traj_output)
    log.info(S('saved trajectory: {}', traj_output))

    if top_output is None:
        top_output = libio.mk_frame_path(traj_output)
    else:
        top_output = Path(top_output)
    
    reimaged[0].save_pdb(top_output.with_suffix('.pdb').str())
    log.info(S('saving frame 0 to: {}', top_output.resolve()))
    return


if __name__ == '__main__':
    maincli()



"""
Copies the trajectory without solvent and extracts the first frame.
"""

from taurenmd.libs import libcli, libmdt


ap = libcli.CustomParser()

ap.add_argument(
    'topology',
    help='The topology structure.',
    )

ap.add_argument(
    'trajectory',
    help='The trajectory',
    )

ap.add_argument(
    '-o',
    '--output_pdb',
    help='Output PDB file.',
    default='production_noHOH.pdb',
    )

ap.add_argument(
    '-d',
    '--traj_output',
    help='The trajectory output.',
    default='production_noHOH.dcd',
    )


def load_args():
    cmd = ap.parse_args()
    return cmd


def maincli():
    
    cmd = load_args()
    main(**vars(cmd))


def main(
        topology,
        trajectory,
        selection=None,
        output_pdb='production_noSol.pdb',
        traj_output='production_noSol.dcd',
        **kwargs
        ):

    trj = libmdt.mdtraj_load_traj(topology, trajectory)
    
    if selection:
        atom_sel = trj.top.select(selection)
        trj.atom_slice(atom_sel, inplace=True)

    trj.remove_solvent(inplace=True)
    trj[0].save_pdb(output_pdb)
    trj.save(traj_output)


if __name__ == '__main__':
    maincli()



def frame_list(len_traj, start=None, stop=None, step=None, flist=None,):
    """
    Createa frame list based on the length of trajectory.

    Parameters
    ----------
    start : int or None, optional
        The start index for the slice object.
    
    stop : int or None, optional
        The end index for the slice object.

    step: int or None, optional
        the step index for the slice object.

    flist : list-like, or comma-separated string, optional
        The list of specific frames.
        Defaults to None.
    """
    if any((start, stop, step)):
        return range(len_traj)[slice(start, stop, step)]

    elif flist:
        try:
            return [int(i) for i in flist.split(',')]
        except AttributeError:
            return [int(i) for i in flist]
        except Exception:
            raise ValueError(
                'Cannot generate list of frames from {}'.format(flist)
                ) from None
    else:
        return range(len_traj)
    

def frame_slice(start=None, stop=None, step=None, ftuple=None):

    if isinstance(ftuple, (list, tuple)) and len(ftuple) == 3:
        return slice(*[int(i) for i in ftuple])
    
    else:
        return slice(start, stop, step)



"""
Input/Output functions for MDTraj.
"""
import mdtraj
import simtk.openmm.app as app

from taurenmd import Path
from taurenmd.libs import libio


def mdtraj_load_traj(topology, trajectory):
    """
    Loads trajectory with MDTraj.

    Parameters
    ----------
    topology
        The topology file.

    traj
        The trajectory file.

    Returns
    -------
    MDTraj trajectory.
    """
    libio.report_input(topology, trajectory)

    topp = Path(topology)
    if topp.suffix == '.cif':
        mol = app.PDBxFile(topp.str())
        top = mdtraj.Topology.from_openmm(mol.topology)
    else:
        top = topp.str()

    mdtrajectory = mdtraj.load(trajectory, top=top)

    return mdtrajectory



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, print_function

import io
import re
from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup


def read(*names, **kwargs):
    with io.open(
            join(dirname(__file__), *names),
            encoding=kwargs.get('encoding', 'utf8')
            ) as fh:
        return fh.read()


setup(
    name='taurenmd',
    version='0.6.0',
    license='GNU GPLv2',
    description='Command-line and library interface for analysis routines in Molecular Dynamics',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
        ),
    author='Joao MC Teixeira',
    author_email='joaomcteixeira@gmail.com',
    url='https://github.com/joaomcteixeira/taurenmd',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Environment :: Console',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Utilities',
        ],
    project_urls={
        'Documentation': 'https://taurenmd.readthedocs.io/',
        'Changelog': 'https://taurenmd.readthedocs.io/en/latest/changelog.html',
        'Issue Tracker': 'https://github.com/joaomcteixeira/taurenmd/issues',
        },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
        ],
    python_requires='>=3, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*',
    install_requires=[
        'bioplottemplates',
        # eg: 'aspectlib==1.1.1', 'six>=1.7',
        ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
        },
    entry_points={
        'console_scripts': [
            'taurenmd = taurenmd.cli:maincli',
            'taurenmd_angle = taurenmd.cli_angle:maincli',
            'taurenmd_dist = taurenmd.cli_distances:maincli',
            'taurenmd_fext = taurenmd.cli_fext:maincli',
            'taurenmd_imagemol = taurenmd.cli_imagemol:maincli',
            'taurenmd_noSol = tauremd.cli_noSol:maincli',
            'taurenmd_report = taurenmd.cli_report:maincli',
            'taurenmd_rmsf = taurenmd.cli_rmsf:maincli',
            'taurenmd_rmsd = taurenmd.cli_rmsd:maincli',
            'taurenmd_trajedit = taurenmd.cli_trajedit:maincli',
            ]
        },
    )



"""
Does something.
"""
import argparse

from taurenmd import log
from taurenmd.libs import libcli, libio  # noqa: F401
from taurenmd.logger import S, T


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    # formatter_class=argparse.RawDescriptionHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(topology, trajectory, **kwargs):
    log.info(T('starting'))
    
    #

    log.info(S('done'))
    return


if __name__ == '__main__':
    maincli()



import logging
import os
from pathlib import Path as _Path

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)

_db = logging.FileHandler('.taurenmd.debug', mode='w')
_db.setLevel(logging.DEBUG)
_db.setFormatter(
    logging.Formatter(
        '%(filename)s:%(name)s:%(funcName)s:%(lineno)d: %(message)s'
        )
    )

_ch = logging.StreamHandler()
_ch.setLevel(logging.INFO)
_ch.setFormatter(logging.Formatter('%(message)s'))

log.addHandler(_db)
log.addHandler(_ch)


CMDFILE = '.taurenmd_cmd_register.log'


class Path(type(_Path())):
    """
    Define a common Path to string interface.

    Avoids using os.fspath around libs.
    """
    def str(self):
        """Return string version of Path."""
        return os.fspath(self)
    
    def myparents(self):
        return self.resolve().parents[0]


__version__ = '0.6.0'



"""
Calculate parameters.
"""
import math

import numpy as np
from MDAnalysis.analysis.rms import RMSD as mdaRMSD
from MDAnalysis.analysis.rms import RMSF as mdaRMSF

from taurenmd import log
from taurenmd.logger import S, T


def mda_rmsd_combined_chains(
        universe,
        frame_slice=None,
        selection='all',
        ref_frame=0,
        ):
    """
    Calculates combined RMSDs.

    Combined RMSDs considers the selection as a whole.

    Parameters
    ----------
    universe
        The MDAnalysis universe.

    frames : str or tuple, optional
        The frames to consider.
        If 'all' considers all frames.
        Otherwise a tuple is needed of the format (start, end, step),
        to be used as a slice object.
        Defaults to `all`.

    selection : str, optional
        The selection upon which calculate the RMSDs.
        Defaults to `all`.

    ref_frames : int, optional
        Defaults to 0.
    
    Returns
    -------
    Numpy Array
        The array containing the calculated RMSDs.
    """
    log.info(T('Calculating RMSDs'))
    
    if frame_slice is None:
        frame_slice = slice(None, None, None)
    elif isinstance(frame_slice, slice):
        pass
    else:
        raise ValueError('frame_slice should be None or slice type')
    
    log.info(S('for selection: {}', selection))
    log.info(S('for trajectory slice: {}', frame_slice))
    
    R = mdaRMSD(
        universe,
        universe,
        select=selection,
        groupselection=None,
        ref_frame=ref_frame,
        verbose=False,
        )
    
    R.run(verbose=False)
    
    # rmsds[:, ii] = R.rmsd[:, 2][self._fslicer]
    
    return R.rmsd[frame_slice, 2]


def mda_rmsf(
        atom_group,
        frame_slice=None,
        ):
    """
    Calculates combined RMSDs.

    Combined RMSDs considers the selection as a whole.

    Parameters
    ----------
    universe
        The MDAnalysis universe.

    frames : str or tuple, optional
        The frames to consider.
        If 'all' considers all frames.
        Otherwise a tuple is needed of the format (start, end, step),
        to be used as a slice object.
        Defaults to `all`.

    selection : str, optional
        The selection upon which calculate the RMSDs.
        Defaults to `all`.

    ref_frames : int, optional
        Defaults to 0.
    
    Returns
    -------
    Numpy Array
        The array containing the calculated RMSDs.
    """
    log.info(T('Calculating RMSFs'))
    
    if frame_slice is None:
        frame_slice = slice(None, None, None)
    elif isinstance(frame_slice, slice):
        pass
    else:
        raise ValueError('frame_slice should be NoneType or Slice')
    
    log.info(S('for trajectory slice: {}', frame_slice))
    
    R = mdaRMSF(
        atom_group,
        start=frame_slice.start,
        stop=frame_slice.stop,
        step=frame_slice.step,
        verbose=False)
    
    R.run()
    
    return R.rmsf


def calc_plane_eq(p1, p2, p3):
    # http://kitchingroup.cheme.cmu.edu/blog/2015/01/18/Equation-of-a-plane-through-three-points/
    v1 = p3 - p1
    v2 = p2 - p1
    cp = np.cross(v1, v2)
    a, b, c = cp
    d = np.dot(cp, p3)
    return a, b, c, d


def calc_angle(a1, b1, c1, a2, b2, c2):
    # from: https://www.geeksforgeeks.org/angle-between-two-planes-in-3d/
    d = (a1 * a2 + b1 * b2 + c1 * c2)
    e1 = math.sqrt(a1 * a1 + b1 * b1 + c1 * c1)
    e2 = math.sqrt(a2 * a2 + b2 * b2 + c2 * c2)
    d = d / (e1 * e2)
    return math.degrees(math.acos(d))



"""
Plane angle calculator.

Calculates the angle of a plane against itself in the reference frame
    along the whole trajectory. The plane is defined by the three
    centres of geometry of tree given selections.
"""
import argparse

from bioplottemplates.plots import param

from taurenmd import log
from taurenmd.libs import libcalc, libcli, libmda, libutil
from taurenmd.logger import S, T


ap = libcli.CustomParser(
    description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    # formatter_class=argparse.RawDescriptionHelpFormatter,
    )

ap.add_argument(
    'topology',
    help='Topology file.',
    type=str,
    )

ap.add_argument(
    'trajectory',
    help=(
        'Trajectory files. If given, multiple trajectories will be'
        'contactenated by order.'
        ),
    nargs='+',
    )

ap.add_argument(
    '-l',
    '--plane-selection',
    help=(
        'Three selections which center of geometry defines the '
        'the plane. For example: \'segid A\' \'segidB \' \'segid C\'.'
        ' The angle will be calculated between this plane along the trajectory '
        ' to the reference frame.'
        ),
    nargs=3
    )

ap.add_argument(
    '-f',
    '--frame',
    help='Calc distances to a frame instead of relative along traj.',
    default=0,
    type=int,
    )

ap.add_argument(
    '-s',
    '--start',
    help='Start frame for slicing.',
    default=None,
    type=int,
    )

ap.add_argument(
    '-e',
    '--stop',
    help='Stop frame for slicing: exclusive',
    default=None,
    type=int,
    )

ap.add_argument(
    '-p',
    '--step',
    help='Step value for slicing',
    default=None,
    type=int,
    )

ap.add_argument(
    '-v',
    '--plotvars',
    help=(
        'Plot variables. '
        'Example: -v xlabel=frames ylabel=RMSD color=red.'
        ),
    nargs='*',
    action=libcli.ParamsToDict,
    )


def load_args():
    """Load user arguments."""
    cmd = ap.parse_args()
    return cmd


def maincli():
    cmd = load_args()
    main(**vars(cmd))
    return


def main(
        topology,
        trajectory,
        plane_selection,
        frame=0,
        start=None,
        stop=None,
        step=None,
        plotvars=None,
        **kwargs
        ):
    log.info(T('calculating angles'))

    u = libmda.mda_load_universe(topology, *list(trajectory))

    frame_slice = libutil.frame_slice(
        start=start,
        stop=stop,
        step=step,
        )
    log.info(S('for slice {}', frame_slice))

    log.info(T('calculating plane eq. for first frame'))
    u.trajectory[0]
    reference_point_1 = u.select_atoms(plane_selection[0]).center_of_geometry()
    reference_point_2 = u.select_atoms(plane_selection[1]).center_of_geometry()
    reference_point_3 = u.select_atoms(plane_selection[2]).center_of_geometry()

    ra, rb, rc, rd = libcalc.calc_plane_eq(
        reference_point_1,
        reference_point_2,
        reference_point_3,
        )
    log.info(S('the equation is {}x + {}y + {}z = {}', ra, rb, rc, rd))
    
    log.info(T('Calculating angles'))
    angles = []
    for fi, ts in zip(
            range(len(u.trajectory))[frame_slice],
            u.trajectory[frame_slice],
            ):

        point1 = u.select_atoms(plane_selection[0]).center_of_geometry()
        point2 = u.select_atoms(plane_selection[1]).center_of_geometry()
        point3 = u.select_atoms(plane_selection[2]).center_of_geometry()
        
        a, b, c, d = libcalc.calc_plane_eq(point1, point2, point3)

        try:
            angles.append(libcalc.calc_angle(ra, rb, rc, a, b, c))
        except ValueError:
            # happens when ValueError: math domain error
            # this is division by zero, means math.acos(d) is 0
            log.info(S('ValueError returned angle 0.0 found'))
            angles.append(0.0)
    
    log.info(S('calculated a total of {} angles.', len(angles)))

    if plotvars is None:
        plotvars = dict()
    
    if 'labels' not in plotvars:
        plotvars['labels'] = 'plane: {}'.format(' and '.join(plane_selection))

    log.info(T('plot params:'))
    for k, v in plotvars.items():
        log.info(S('{} = {!r}', k, v))

    param.plot(
        list(range(len(u.trajectory))[frame_slice]),
        angles,
        **plotvars,
        )

    log.info(S('done'))
    return


if __name__ == '__main__':
    maincli()
